@@ -51,22 +51,20 @@ static void rb_metadata_finalize (GObject *object);
 typedef GstElement *(*RBAddTaggerElem) (RBMetaData *, GstElement *);
 
 /*
- * The list of mine type prefixes for files that shouldn't display errors about being non-audio.
+ * The list of mime-type prefixes for files that shouldn't display errors about being non-audio.
  * Useful for people who have cover art, et cetera, in their music directories
  */
 const char * ignore_mime_types[] = {
 	"image/",
 	"text/",
-	"application/xml",
-	"application/zip",
-	"application/x-executable",
-	"application/x-ms-dos-executable",
-	"application/x-bzip",
-	"application/x-gzip",
-	"application/pdf",
-	"application/x-rar",
-	"application/msword",
-	"application/octet-stream"
+	"application/",
+};
+
+const char * non_ignore_mime_types[] = {
+	"application/x-id3",
+	"application/ogg",
+	"application/x-apetag",
+	"application/x-3gp"
 };
 
 /*
@@ -516,7 +514,6 @@ rb_metadata_gst_tag_to_field (const char *tag)
 		return RB_METADATA_FIELD_ALBUM_GAIN;
 	else if (!strcmp (tag, GST_TAG_ALBUM_PEAK))
 		return RB_METADATA_FIELD_ALBUM_PEAK;
-#ifdef GST_TAG_MUSICBRAINZ_TRACKID
 	else if (!strcmp (tag, GST_TAG_MUSICBRAINZ_TRACKID))
 		return RB_METADATA_FIELD_MUSICBRAINZ_TRACKID;
 	else if (!strcmp (tag, GST_TAG_MUSICBRAINZ_ARTISTID))
@@ -525,10 +522,10 @@ rb_metadata_gst_tag_to_field (const char *tag)
 		return RB_METADATA_FIELD_MUSICBRAINZ_ALBUMID;
 	else if (!strcmp (tag, GST_TAG_MUSICBRAINZ_ALBUMARTISTID))
 		return RB_METADATA_FIELD_MUSICBRAINZ_ALBUMARTISTID;
-	else if (!strcmp (tag, GST_TAG_MUSICBRAINZ_SORTNAME))
+	else if (!strcmp (tag, GST_TAG_ARTIST_SORTNAME))
 		return RB_METADATA_FIELD_ARTIST_SORTNAME;
-
-#endif
+	else if (!strcmp (tag, GST_TAG_ALBUM_SORTNAME))
+		return RB_METADATA_FIELD_ALBUM_SORTNAME;
 	else
 		return -1;
 }
@@ -588,7 +585,6 @@ rb_metadata_gst_field_to_gst_tag (RBMetaDataField field)
 		return GST_TAG_ALBUM_GAIN;
 	case RB_METADATA_FIELD_ALBUM_PEAK:
 		return GST_TAG_ALBUM_PEAK;
-#ifdef GST_TAG_MUSICBRAINZ_TRACKID
 	case RB_METADATA_FIELD_MUSICBRAINZ_TRACKID:
 		return GST_TAG_MUSICBRAINZ_TRACKID;
 	case RB_METADATA_FIELD_MUSICBRAINZ_ARTISTID:
@@ -598,8 +594,9 @@ rb_metadata_gst_field_to_gst_tag (RBMetaDataField field)
 	case RB_METADATA_FIELD_MUSICBRAINZ_ALBUMARTISTID:
 		return GST_TAG_MUSICBRAINZ_ALBUMARTISTID;
 	case RB_METADATA_FIELD_ARTIST_SORTNAME:
-		return GST_TAG_MUSICBRAINZ_SORTNAME;
-#endif
+		return GST_TAG_ARTIST_SORTNAME;
+	case RB_METADATA_FIELD_ALBUM_SORTNAME:
+		return GST_TAG_ALBUM_SORTNAME;
 	default:
 		return NULL;
 	}
@@ -786,8 +783,9 @@ rb_metadata_gst_new_decoded_pad_cb (GstElement *decodebin, GstPad *pad, gboolean
 	} else {
 		GstPad *sink_pad;
 
-		sink_pad = gst_element_get_pad (md->priv->sink, "sink");
+		sink_pad = gst_element_get_static_pad (md->priv->sink, "sink");
 		gst_pad_link (pad, sink_pad);
+		gst_object_unref (sink_pad);
 
 		/* is this pad audio? */
 		structure = gst_caps_get_structure (caps, 0);
@@ -965,7 +963,7 @@ rb_metadata_event_loop (RBMetaData *md, GstElement *element, gboolean block)
 		GstMessage *message;
 
 		if (block)
-			message = gst_bus_poll (bus, GST_MESSAGE_ANY, -1);
+			message = gst_bus_timed_pop (bus, GST_CLOCK_TIME_NONE);
 		else
 			message = gst_bus_pop (bus);
 
@@ -980,6 +978,23 @@ rb_metadata_event_loop (RBMetaData *md, GstElement *element, gboolean block)
 	gst_object_unref (bus);
 }
 
+static gboolean
+ignore_mimetype (const char *type)
+{
+	guint i;
+
+	for (i = 0; i < G_N_ELEMENTS (non_ignore_mime_types); i++) {
+		if (g_str_equal (type, non_ignore_mime_types[i]) != FALSE)
+			return FALSE;
+	}
+	for (i = 0; i < G_N_ELEMENTS (ignore_mime_types); i++) {
+		if (g_str_has_prefix (type, ignore_mime_types[i]) != FALSE)
+			return TRUE;
+	}
+
+	return FALSE;
+}
+
 void
 rb_metadata_load (RBMetaData *md,
 		  const char *uri,
@@ -1079,7 +1094,7 @@ rb_metadata_load (RBMetaData *md,
 	       change_timeout < 5) {
 		GstMessage *msg;
 
-		msg = gst_bus_poll (bus, GST_MESSAGE_ANY, 1 * GST_SECOND);
+		msg = gst_bus_timed_pop (bus, 1 * GST_SECOND);
 		if (msg) {
 			rb_metadata_bus_handler (bus, msg, md);
 			gst_message_unref (msg);
@@ -1151,17 +1166,11 @@ rb_metadata_load (RBMetaData *md,
 	if ((md->priv->has_video || !md->priv->has_audio) &&
 	    (md->priv->has_non_audio || !md->priv->handoff)) {
 		gboolean ignore = FALSE;
-		int i;
 
 		if (md->priv->has_video) {
 			ignore = TRUE;
 		} else {
-			for (i = 0; i < G_N_ELEMENTS (ignore_mime_types); i++) {
-				if (g_str_has_prefix (md->priv->type, ignore_mime_types[i])) {
-					ignore = TRUE;
-					break;
-				}
-			}
+			ignore = ignore_mimetype (md->priv->type);
 		}
 
 		if (!ignore) {
