@@ -10,7 +10,7 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
  *  GStreamer plugins to be used and distributed together with GStreamer
  *  and Rhythmbox. This permission is above and beyond the permissions granted
  *  by the GPL license by which Rhythmbox is covered. If you modify this code
@@ -44,21 +44,18 @@
 
 #include <string.h>
 #include <math.h>
+#include <unistd.h>
 
 #include <glib/gi18n.h>
 #include <glib/gstdio.h>
 #include <gtk/gtk.h>
-#include <glade/glade.h>
 
 #include <gconf/gconf-value.h>
 
 #include <totem-pl-parser.h>
 
-#include "rb-soup-compat.h"
 #include <libsoup/soup.h>
 
-#include "md5.h"
-
 #include "eel-gconf-extensions.h"
 
 #include "rb-proxy-config.h"
@@ -68,7 +65,6 @@
 #include "rb-lastfm-source.h"
 
 #include "rhythmdb-query-model.h"
-#include "rb-glade-helpers.h"
 #include "rb-stock-icons.h"
 #include "rb-entry-view.h"
 #include "rb-property-view.h"
@@ -78,13 +74,14 @@
 #include "rb-dialog.h"
 #include "rb-debug.h"
 #include "eel-gconf-extensions.h"
+#include "gedit-message-area.h"
 #include "rb-shell-player.h"
 #include "rb-play-order.h"
 #include "rb-lastfm-play-order.h"
 
 #define LASTFM_URL "ws.audioscrobbler.com"
 #define RB_LASTFM_PLATFORM "linux"
-#define RB_LASTFM_VERSION "1.3.1.1"
+#define RB_LASTFM_VERSION "1.5"
 
 #define USER_AGENT "Rhythmbox/" VERSION
 
@@ -140,6 +137,7 @@ static void rb_lastfm_source_get_property (GObject *object,
 
 static void rb_lastfm_source_songs_view_sort_order_changed_cb (RBEntryView *view,
 							       RBLastfmSource *source);
+/* source-specific methods */
 
 static void rb_lastfm_source_drag_cb (GtkWidget *widget,
 				      GdkDragContext *dc,
@@ -195,6 +193,10 @@ static const char* const radio_options[][3] = {
 	{N_("Tag radio"), "lastfm://globaltags/%s", N_("Tracks tagged with %s")},
 	{N_("Artist Fan radio"), "lastfm://artist/%s/fans", N_("Artists liked by fans of %s")},
 	{N_("Group radio"), "lastfm://group/%s", N_("Tracks liked by the %s group")},
+	{N_("Neighbour radio"), "lastfm://user/%s/neighbours", N_("%s's Neighbour Radio")},
+	{N_("Personal radio"), "lastfm://user/%s/personal", N_("%s's Personal Radio")},
+	{N_("Loved tracks"), "lastfm://user/%s/loved", N_("%s's Loved Tracks")},
+	{N_("Recommended tracks"), "lastfm://user/%s/recommended/100", N_("Tracks recommended to %s")},
 	{NULL, NULL, NULL}
 };
 
@@ -209,10 +211,13 @@ typedef struct
 
 struct RBLastfmSourcePrivate
 {
+	GtkWidget *main_box;
 	GtkWidget *paned;
+	GtkWidget *message_area;
 	GtkWidget *txtbox;
 	GtkWidget *typecombo;
-	RhythmDB *db;
+	GtkWidget *config_widget;
+	RhythmDB  *db;
 
 	GtkActionGroup *action_group;
 
@@ -227,7 +232,7 @@ struct RBLastfmSourcePrivate
 	RBPlayOrder *play_order;
 
 	RhythmDBQueryModel *query_model;
-	int tracks_remaining;
+	RhythmDBEntry *last_entry;
 
 	gboolean subscriber;
 	char *base_url;
@@ -241,6 +246,9 @@ struct RBLastfmSourcePrivate
 		STATION_FAILED
 	} state;
 
+	guint notification_username_id;
+	guint notification_password_id;
+
 	GQueue *action_queue;
 	gboolean request_outstanding;
 	const char *request_description;
@@ -267,14 +275,14 @@ enum
 {
 	PROP_0,
 	PROP_ENTRY_TYPE,
-	PROP_TRACK_ENTRY_TYPE,
+	PROP_STATION_ENTRY_TYPE,
 	PROP_PROXY_CONFIG,
 	PROP_PLAY_ORDER
 };
 
 static GtkActionEntry rb_lastfm_source_actions [] =
 {
-	{ "LastfmLoveSong", GTK_STOCK_ADD, N_("Love"), NULL,
+	{ "LastfmLoveSong", "emblem-favorite", N_("Love"), NULL,
 	  N_("Mark this song as loved"),
 	  G_CALLBACK (rb_lastfm_source_love_track) },
 	{ "LastfmBanSong", GTK_STOCK_CANCEL, N_("Ban"), NULL,
@@ -325,14 +333,14 @@ rb_lastfm_source_class_init (RBLastfmSourceClass *klass)
 					 PROP_ENTRY_TYPE,
 					 g_param_spec_boxed ("entry-type",
 							     "Entry type",
-							     "Entry type for last.fm stations",
+							     "Entry type for last.fm tracks",
 							     RHYTHMDB_TYPE_ENTRY_TYPE,
 							     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_install_property (object_class,
-					 PROP_TRACK_ENTRY_TYPE,
-					 g_param_spec_boxed ("track-entry-type",
+					 PROP_STATION_ENTRY_TYPE,
+					 g_param_spec_boxed ("station-entry-type",
 							     "Entry type",
-							     "Entry type for last.fm tracks",
+							     "Entry type for last.fm stations",
 							     RHYTHMDB_TYPE_ENTRY_TYPE,
 							     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_install_property (object_class,
@@ -351,12 +359,44 @@ rb_lastfm_source_class_init (RBLastfmSourceClass *klass)
 	g_type_class_add_private (klass, sizeof (RBLastfmSourcePrivate));
 }
 
+static void
+on_gconf_changed_cb (GConfClient    *client,
+		     guint           cnxn_id,
+		     GConfEntry     *entry,
+		     RBLastfmSource *source)
+{
+	rb_debug ("GConf key updated: \"%s\"", entry->key);
+
+
+	if (source->priv->state == CONNECTED) {
+		return;
+	}
+
+	if (strcmp (entry->key, CONF_AUDIOSCROBBLER_USERNAME) == 0
+	    || strcmp (entry->key, CONF_AUDIOSCROBBLER_PASSWORD) == 0) {
+		source->priv->state = NOT_CONNECTED;
+		queue_handshake (source);
+	} else {
+		rb_debug ("Unhandled GConf key updated: \"%s\"", entry->key);
+	}
+}
+
 static void
 rb_lastfm_source_init (RBLastfmSource *source)
 {
 	source->priv = G_TYPE_INSTANCE_GET_PRIVATE ((source), RB_TYPE_LASTFM_SOURCE,  RBLastfmSourcePrivate);
 
 	source->priv->action_queue = g_queue_new ();
+
+	source->priv->notification_username_id =
+		eel_gconf_notification_add (CONF_AUDIOSCROBBLER_USERNAME,
+					    (GConfClientNotifyFunc) on_gconf_changed_cb,
+					    source);
+	source->priv->notification_password_id =
+		eel_gconf_notification_add (CONF_AUDIOSCROBBLER_PASSWORD,
+					    (GConfClientNotifyFunc) on_gconf_changed_cb,
+					    source);
+
 }
 
 static void
@@ -392,6 +432,15 @@ rb_lastfm_source_dispose (GObject *object)
 		source->priv->query_model = NULL;
 	}
 
+	if (source->priv->notification_username_id != 0) {
+		eel_gconf_notification_remove (source->priv->notification_username_id);
+		source->priv->notification_username_id = 0;
+	}
+	if (source->priv->notification_password_id != 0) {
+		eel_gconf_notification_remove (source->priv->notification_password_id);
+		source->priv->notification_password_id = 0;
+	}
+
 	/* kill entries here? */
 
 	G_OBJECT_CLASS (rb_lastfm_source_parent_class)->dispose (object);
@@ -422,7 +471,6 @@ rb_lastfm_source_constructor (GType type, guint n_construct_properties,
 	RBLastfmSourceClass *klass;
 	RBShell *shell;
 	GtkWidget *editor_vbox;
-	GtkWidget *main_box;
 	GtkWidget *editor_box;
 	GtkWidget *add_button;
 	GtkWidget *instructions;
@@ -480,12 +528,16 @@ rb_lastfm_source_constructor (GType type, guint n_construct_properties,
 	gtk_combo_box_set_active (GTK_COMBO_BOX (source->priv->typecombo), 0);
 
 	source->priv->txtbox = gtk_entry_new ();
+	g_signal_connect_object (G_OBJECT (source->priv->txtbox),
+				 "activate",
+				 G_CALLBACK (rb_lastfm_source_add_station_cb),
+				 source, 0);
 
-	gtk_box_pack_end_defaults (GTK_BOX (editor_box), add_button);
-	gtk_box_pack_end_defaults (GTK_BOX (editor_box), source->priv->txtbox);
-	gtk_box_pack_start_defaults (GTK_BOX (editor_box), source->priv->typecombo);
-	gtk_box_pack_end_defaults (GTK_BOX (editor_vbox), editor_box);
-	gtk_box_pack_end_defaults (GTK_BOX (editor_vbox), instructions);
+	gtk_box_pack_end (GTK_BOX (editor_box), add_button, TRUE, TRUE, 0);
+	gtk_box_pack_end (GTK_BOX (editor_box), source->priv->txtbox, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (editor_box), source->priv->typecombo, TRUE, TRUE, 0);
+	gtk_box_pack_end (GTK_BOX (editor_vbox), editor_box, TRUE, TRUE, 0);
+	gtk_box_pack_end (GTK_BOX (editor_vbox), instructions, TRUE, TRUE, 0);
 
 	source->priv->paned = gtk_vpaned_new ();
 
@@ -537,10 +589,10 @@ rb_lastfm_source_constructor (GType type, guint n_construct_properties,
 	gtk_paned_pack1 (GTK_PANED (source->priv->paned), GTK_WIDGET (source->priv->stations), TRUE, TRUE);
 	gtk_paned_pack2 (GTK_PANED (source->priv->paned), GTK_WIDGET (source->priv->tracks), TRUE, TRUE);
 
-	main_box = gtk_vbox_new (FALSE, 5);
-	gtk_box_pack_start (GTK_BOX (main_box), editor_vbox, FALSE, FALSE, 5);
-	gtk_box_pack_start_defaults (GTK_BOX (main_box), source->priv->paned);
-	gtk_container_add (GTK_CONTAINER (source), main_box);
+	source->priv->main_box = gtk_vbox_new (FALSE, 5);
+	gtk_box_pack_start (GTK_BOX (source->priv->main_box), editor_vbox, FALSE, FALSE, 5);
+	gtk_box_pack_start (GTK_BOX (source->priv->main_box), source->priv->paned, TRUE, TRUE, 0);
+	gtk_container_add (GTK_CONTAINER (source), source->priv->main_box);
 
 	gtk_widget_show_all (GTK_WIDGET (source));
 
@@ -570,7 +622,6 @@ rb_lastfm_source_constructor (GType type, guint n_construct_properties,
 	g_object_unref (station_query_model);
 
 	source->priv->query_model = rhythmdb_query_model_new_empty (source->priv->db);
-	source->priv->tracks_remaining = 0;
 	rb_entry_view_set_model (source->priv->tracks, source->priv->query_model);
 	
 	g_object_set (source, "query-model", source->priv->query_model, NULL);
@@ -588,11 +639,11 @@ rb_lastfm_source_set_property (GObject *object,
 
 	switch (prop_id) {
 	case PROP_ENTRY_TYPE:
-		source->priv->station_entry_type = g_value_get_boxed (value);
-		break;
-	case PROP_TRACK_ENTRY_TYPE:
 		source->priv->track_entry_type = g_value_get_boxed (value);
 		break;
+	case PROP_STATION_ENTRY_TYPE:
+		source->priv->station_entry_type = g_value_get_boxed (value);
+		break;
 	case PROP_PROXY_CONFIG:
 		source->priv->proxy_config = g_value_get_object (value);
 		g_object_ref (G_OBJECT (source->priv->proxy_config));
@@ -613,11 +664,11 @@ rb_lastfm_source_get_property (GObject *object,
 
 	switch (prop_id) {
 	case PROP_ENTRY_TYPE:
-		g_value_set_boxed (value, source->priv->station_entry_type);
-		break;
-	case PROP_TRACK_ENTRY_TYPE:
 		g_value_set_boxed (value, source->priv->track_entry_type);
 		break;
+	case PROP_STATION_ENTRY_TYPE:
+		g_value_set_boxed (value, source->priv->station_entry_type);
+		break;
 	case PROP_PLAY_ORDER:
 		g_value_set_object (value, source->priv->play_order);
 		break;
@@ -642,7 +693,8 @@ destroy_track_data (RhythmDBEntry *entry, gpointer meh)
 
 
 RBSource *
-rb_lastfm_source_new (RBShell *shell)
+rb_lastfm_source_new (RBPlugin *plugin,
+		      RBShell  *shell)
 {
 	RBSource *source;
 	RBProxyConfig *proxy_config;
@@ -676,14 +728,15 @@ rb_lastfm_source_new (RBShell *shell)
 	g_object_get (G_OBJECT (shell), "proxy-config", &proxy_config, NULL);
 
 	source = RB_SOURCE (g_object_new (RB_TYPE_LASTFM_SOURCE,
+					  "plugin", plugin,
 					  "name", _("Last.fm"),
 					  "shell", shell,
-					  "entry-type", station_entry_type,
-					  "track-entry-type", track_entry_type,
+					  "station-entry-type", station_entry_type,
+					  "entry-type", track_entry_type,
 					  "proxy-config", proxy_config,
 					  "source-group", RB_SOURCE_GROUP_LIBRARY,
 					  NULL));
-	rb_shell_register_entry_type_for_source (shell, source, station_entry_type);
+
 	rb_shell_register_entry_type_for_source (shell, source, track_entry_type);
 
 	g_object_unref (db);
@@ -710,6 +763,165 @@ impl_get_entry_view (RBSource *asource)
 	return source->priv->tracks;
 }
 
+static void
+set_message_area_text_and_icon (RBLastfmSource *source,
+				const char     *icon_stock_id,
+				const char     *primary_text,
+				const char     *secondary_text)
+{
+	GtkWidget *hbox_content;
+	GtkWidget *image;
+	GtkWidget *vbox;
+	char      *primary_markup;
+	char      *secondary_markup;
+	GtkWidget *primary_label;
+	GtkWidget *secondary_label;
+
+	hbox_content = gtk_hbox_new (FALSE, 8);
+	gtk_widget_show (hbox_content);
+
+	image = gtk_image_new_from_stock (icon_stock_id, GTK_ICON_SIZE_DIALOG);
+	gtk_widget_show (image);
+	gtk_box_pack_start (GTK_BOX (hbox_content), image, FALSE, FALSE, 0);
+	gtk_misc_set_alignment (GTK_MISC (image), 0.5, 0);
+
+	vbox = gtk_vbox_new (FALSE, 6);
+	gtk_widget_show (vbox);
+	gtk_box_pack_start (GTK_BOX (hbox_content), vbox, TRUE, TRUE, 0);
+
+	primary_markup = g_strdup_printf ("<b>%s</b>", primary_text);
+	primary_label = gtk_label_new (primary_markup);
+	g_free (primary_markup);
+	gtk_widget_show (primary_label);
+	gtk_box_pack_start (GTK_BOX (vbox), primary_label, TRUE, TRUE, 0);
+	gtk_label_set_use_markup (GTK_LABEL (primary_label), TRUE);
+	gtk_label_set_line_wrap (GTK_LABEL (primary_label), TRUE);
+	gtk_misc_set_alignment (GTK_MISC (primary_label), 0, 0.5);
+	GTK_WIDGET_SET_FLAGS (primary_label, GTK_CAN_FOCUS);
+	gtk_label_set_selectable (GTK_LABEL (primary_label), TRUE);
+
+  	if (secondary_text != NULL) {
+  		secondary_markup = g_strdup_printf ("<small>%s</small>",
+  						    secondary_text);
+		secondary_label = gtk_label_new (secondary_markup);
+		g_free (secondary_markup);
+		gtk_widget_show (secondary_label);
+		gtk_box_pack_start (GTK_BOX (vbox), secondary_label, TRUE, TRUE, 0);
+		GTK_WIDGET_SET_FLAGS (secondary_label, GTK_CAN_FOCUS);
+		gtk_label_set_use_markup (GTK_LABEL (secondary_label), TRUE);
+		gtk_label_set_line_wrap (GTK_LABEL (secondary_label), TRUE);
+		gtk_label_set_selectable (GTK_LABEL (secondary_label), TRUE);
+		gtk_misc_set_alignment (GTK_MISC (secondary_label), 0, 0.5);
+	}
+
+	gtk_widget_show (source->priv->message_area);
+	gedit_message_area_set_contents (GEDIT_MESSAGE_AREA (source->priv->message_area),
+					 hbox_content);
+}
+
+static void
+set_message_area (RBLastfmSource *source,
+		  GtkWidget      *area)
+{
+	if (source->priv->message_area == area) {
+		return;
+	}
+
+	if (source->priv->message_area) {
+		gtk_widget_destroy (source->priv->message_area);
+	}
+	source->priv->message_area = area;
+
+	if (area == NULL) {
+		return;
+	}
+
+	gtk_box_pack_end (GTK_BOX (source->priv->main_box),
+			  source->priv->message_area,
+			  FALSE, FALSE, 0);
+#if 0
+	gtk_box_reorder_child (GTK_BOX (source->priv->view_box),
+			       source->priv->message_area, 0);
+#endif
+	g_object_add_weak_pointer (G_OBJECT (source->priv->message_area),
+				   (gpointer) &(source->priv->message_area));
+}
+
+static void
+on_message_area_response (GeditMessageArea *area,
+			  int               response_id,
+			  RBLastfmSource   *source)
+{
+	RBPlugin  *plugin;
+	GtkWidget *dialog;
+
+	g_object_get (source, "plugin", &plugin, NULL);
+	dialog = rb_plugin_create_configure_dialog (plugin);
+	g_object_unref (plugin);
+}
+
+static void
+show_error_message (RBLastfmSource *source,
+		    const char     *primary_text,
+		    const char     *secondary_text)
+{
+	GtkWidget *area;
+
+	if (source->priv->message_area != NULL) {
+		return;
+	}
+
+	area = gedit_message_area_new_with_buttons (_("Account Settings"),
+						    GTK_RESPONSE_ACCEPT,
+						    NULL);
+	set_message_area (source, area);
+	set_message_area_text_and_icon (source,
+					"gtk-dialog-error",
+					primary_text,
+					secondary_text);
+	g_signal_connect (area,
+			  "response",
+			  G_CALLBACK (on_message_area_response),
+			  source);
+}
+
+static void
+update_message_area (RBLastfmSource *source)
+{
+	char *primary_text;
+	char *secondary_text;
+
+	primary_text = NULL;
+	secondary_text = NULL;
+
+	switch (source->priv->state) {
+	case LOGIN_FAILED:
+		primary_text = g_strdup (_("Account details are needed before you can connect.  Check your settings."));
+		break;
+
+	case BANNED:
+		primary_text = g_strdup (_("This version of Rhythmbox has been banned from Last.fm."));
+		break;
+
+	case STATION_FAILED:
+		primary_text = g_strdup (_("Unable to connect"));
+		secondary_text = g_strdup (source->priv->station_failed_reason);
+		break;
+
+	case NOT_CONNECTED:
+	case CONNECTED:
+		set_message_area (source, NULL);
+		break;
+	default:
+		g_assert_not_reached ();
+		break;
+	}
+
+	if (primary_text != NULL) {
+		show_error_message (source, primary_text, secondary_text);
+	}
+}
+
 static void
 impl_get_status (RBSource *asource, char **text, char **progress_text, float *progress)
 {
@@ -718,17 +930,9 @@ impl_get_status (RBSource *asource, char **text, char **progress_text, float *pr
 
 	switch (source->priv->state) {
 	case LOGIN_FAILED:
-		*text = g_strdup (_("Could not log in to Last.fm.  Check your username and password."));
-		break;
-
 	case BANNED:
-		*text = g_strdup (_("This version of Rhythmbox has been banned from Last.fm."));
-		break;
-
 	case STATION_FAILED:
-		*text = g_strdup (source->priv->station_failed_reason);
 		break;
-
 	case NOT_CONNECTED:
 	case CONNECTED:
 		g_object_get (asource, "query-model", &model, NULL);
@@ -737,6 +941,8 @@ impl_get_status (RBSource *asource, char **text, char **progress_text, float *pr
 		break;
 	}
 
+	update_message_area (source);
+
 	rb_streaming_source_get_progress (RB_STREAMING_SOURCE (source), progress_text, progress);
 	
 	/* pulse progressbar if there's something going on */
@@ -763,9 +969,6 @@ impl_delete (RBSource *asource)
 		track = (RhythmDBEntry *)l->data;
 		track_data = RHYTHMDB_ENTRY_GET_TYPE_DATA (track, RBLastfmTrackEntryData);
 
-		if (track_data->played == FALSE) {
-			source->priv->tracks_remaining--;
-		}
 		rhythmdb_entry_delete (source->priv->db, track);
 	}
 	rhythmdb_commit (source->priv->db);
@@ -915,31 +1118,27 @@ playing_song_changed_cb (RBShellPlayer *player,
 		track_data = RHYTHMDB_ENTRY_GET_TYPE_DATA (entry, RBLastfmTrackEntryData);
 		if (track_data->played == FALSE) {
 
-			if (source->priv->current_station != NULL) {
-				source->priv->tracks_remaining--;
-				if (source->priv->tracks_remaining < 1) {
+			if (source->priv->current_station != NULL && entry == source->priv->last_entry) {
 
-					GList *sel;
-					RhythmDBEntry *selected_station = NULL;
-					/* if a new station has been selected, change station before
-					 * refreshing the playlist.
-					 */
-					sel = rb_entry_view_get_selected_entries (source->priv->stations);
-					if (sel != NULL) {
-						selected_station = (RhythmDBEntry *)sel->data;
-						if (selected_station != source->priv->current_station) {
-							rb_debug ("changing to station %s",
-								  rhythmdb_entry_get_string (selected_station, RHYTHMDB_PROP_LOCATION));
-							queue_change_station (source, selected_station);
-						}
-						queue_get_playlist (source, selected_station);
-					} else {
-						queue_get_playlist (source, source->priv->current_station);
+				GList *sel;
+				RhythmDBEntry *selected_station = NULL;
+				/* if a new station has been selected, change station before
+				 * refreshing the playlist.
+				 */
+				sel = rb_entry_view_get_selected_entries (source->priv->stations);
+				if (sel != NULL) {
+					selected_station = (RhythmDBEntry *)sel->data;
+					if (selected_station != source->priv->current_station) {
+						rb_debug ("changing to station %s",
+							  rhythmdb_entry_get_string (selected_station, RHYTHMDB_PROP_LOCATION));
+						queue_change_station (source, selected_station);
 					}
-					g_list_foreach (sel, (GFunc)rhythmdb_entry_unref, NULL);
-					g_list_free (sel);
-
+					queue_get_playlist (source, selected_station);
+				} else {
+					queue_get_playlist (source, source->priv->current_station);
 				}
+				g_list_foreach (sel, (GFunc)rhythmdb_entry_unref, NULL);
+				g_list_free (sel);
 			}
 			track_data->played = TRUE;
 		}
@@ -975,6 +1174,8 @@ rb_lastfm_source_ban_track (GtkAction *run_action, RBLastfmSource *source)
 	gtk_action_set_sensitive (action, FALSE);
 	action = gtk_action_group_get_action (source->priv->action_group, "LastfmBanSong");
 	gtk_action_set_sensitive (action, FALSE);
+
+	rb_shell_player_do_next (source->priv->shell_player, NULL);
 }
 
 static void
@@ -1058,10 +1259,8 @@ rb_lastfm_source_station_selection_cb (RBEntryView *stations,
 		rb_debug ("station %s selected",
 			  rhythmdb_entry_get_string (selected, RHYTHMDB_PROP_LOCATION));
 
-		/* if we don't have any more tracks to play, or this is the first station
-		 * selection, update the playlist.
-		 */
-		if (source->priv->tracks_remaining < 1) {
+		/* if this is the first station selected, update the playlist */
+		if (source->priv->last_entry == NULL) {
 			queue_change_station (source, selected);
 			queue_get_playlist (source, selected);
 		}
@@ -1124,7 +1323,7 @@ rb_lastfm_source_title_from_uri (const char *uri)
 	}
 
 	g_strfreev (data);
-	unesc_title = gnome_vfs_unescape_string (title, NULL);
+	unesc_title = g_uri_unescape_string (title, NULL);
 	g_free (title);
 	return unesc_title;
 }
@@ -1256,33 +1455,14 @@ free_action (RBLastfmAction *action)
 	g_free (action);
 }
 
-#if defined(HAVE_LIBSOUP_2_4)
 static void
 http_response_cb (SoupSession *session, SoupMessage *req, gpointer user_data)
-#else
-static void
-http_response_cb (SoupMessage *req, gpointer user_data)
-#endif
 {
 	RBLastfmAction *action = (RBLastfmAction *)user_data;
 	RBLastfmSource *source = action->source;
 	char *free_body;
 	const char *body;
 
-#if defined(HAVE_LIBSOUP_2_2)
-
-	if ((req->response).body == NULL) {
-		body = NULL;
-		free_body = NULL;
-		rb_debug ("server failed to respond");
-	} else {
-		free_body = g_malloc0 ((req->response).length + 1);
-		memcpy (free_body, (req->response).body, (req->response).length);
-		g_strstrip (free_body);
-
-		body = free_body;
-	}
-#else
 	free_body = NULL;
 	if (req->response_body->length == 0) {
 		rb_debug ("server failed to respond");
@@ -1290,7 +1470,6 @@ http_response_cb (SoupMessage *req, gpointer user_data)
 	} else {
 		body = req->response_body->data;
 	}
-#endif
 
 	/* call the action's response handler */
 	if (action->handle_response != NULL) {
@@ -1405,7 +1584,6 @@ queue_action (RBLastfmSource *source,
 
 /* common protocol utility stuff */
 
-#if defined(HAVE_LIBSOUP_2_4)
 static char *
 auth_challenge (RBLastfmSource *source)
 {
@@ -1415,46 +1593,33 @@ auth_challenge (RBLastfmSource *source)
 	 */
 	return g_strdup_printf ("%ld", time (NULL));
 }
-#endif
 
 static gchar *
 mkmd5 (char *string, char *string2)
 {
-	md5_state_t md5state;
-	guchar md5pword[16];
-	gchar md5_response[33];
-
-	int j = 0;
-
-	memset (md5_response, 0, sizeof (md5_response));
-
-	md5_init (&md5state);
-	md5_append (&md5state, (unsigned char*)string, strlen (string));
+	GChecksum *checksum;
+	gchar *md5_result;
+	
+	checksum = g_checksum_new(G_CHECKSUM_MD5);	
+	g_checksum_update(checksum, (guchar *)string, -1);
+	
 	if (string2 != NULL) {
-		md5_append (&md5state, (unsigned char*)string2, strlen (string2));
+		g_checksum_update(checksum, (guchar *)string2, -1);		
 	}
-	md5_finish (&md5state, md5pword);
-
-	for (j = 0; j < 16; j++) {
-		char a[3];
-		sprintf (a, "%02x", md5pword[j]);
-		md5_response[2*j] = a[0];
-		md5_response[2*j+1] = a[1];
-	}
-
-	return (g_strdup (md5_response));
+	
+	md5_result = g_strdup(g_checksum_get_string(checksum));
+	g_checksum_free(checksum);
+	
+	return (md5_result);
 }
 
 static gboolean
 station_is_subscriber_only (const char *uri)
 {
-	/* personal and loved-tracks radio */
-	if (g_str_has_prefix (uri, "lastfm://user/")) {
-		if (g_str_has_suffix (uri, "/personal"))
-			return TRUE;
-
-		if (g_str_has_suffix (uri, "/loved"))
-			return TRUE;
+	/* loved-tracks radio */
+	if (g_str_has_prefix (uri, "lastfm://user/") &&
+	    g_str_has_suffix (uri, "/loved")) {
+		return TRUE;
 	}
 
 	/* user tag radio */
@@ -1565,7 +1730,9 @@ handle_handshake_response (RBLastfmSource *source, const char *body, RhythmDBEnt
 	pieces = g_strsplit (body, "\n", 0);
 	for (i = 0; pieces[i] != NULL; i++) {
 		gchar **values = g_strsplit (pieces[i], "=", 2);
-		if (strcmp (values[0], "session") == 0) {
+		if (values[0] == NULL) {
+			rb_debug ("unexpected response content: %s", pieces[i]);
+		} else if (strcmp (values[0], "session") == 0) {
 			if (strcmp (values[1], "FAILED") == 0) {
 				source->priv->state = LOGIN_FAILED;
 				rb_debug ("login failed");
@@ -1600,6 +1767,10 @@ handle_handshake_response (RBLastfmSource *source, const char *body, RhythmDBEnt
 
 	g_strfreev (pieces);
 
+	if (source->priv->state != CONNECTED) {
+		return;
+	}
+
 	/* create default stations */
 	username = eel_gconf_get_string (CONF_AUDIOSCROBBLER_USERNAME);
 	if (username != NULL) {
@@ -1665,7 +1836,9 @@ create_station_request (RBLastfmSource *source, RhythmDBEntry *entry)
 		return NULL;
 	}
 
-	lastfm_url = gnome_vfs_escape_string (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION));
+	lastfm_url = g_uri_escape_string (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION),
+					  G_URI_RESERVED_CHARS_ALLOWED_IN_PATH,
+					  FALSE);
 
 	url = g_strdup_printf("http://%s%s/adjust.php?session=%s&url=%s&debug=0",
 			      source->priv->base_url ? source->priv->base_url : LASTFM_URL,
@@ -1716,7 +1889,9 @@ handle_station_response (RBLastfmSource *source, const char *body, RhythmDBEntry
 	for (i = 0; pieces[i] != NULL; i++) {
 		gchar **values = g_strsplit (pieces[i], "=", 2);
 
-		if (strcmp (values[0], "response") == 0) {
+		if (values[0] == NULL) {
+			rb_debug ("unexpected response content: %s", pieces[i]);
+		} else if (strcmp (values[0], "response") == 0) {
 			if (source->priv->current_station != NULL) {
 				rhythmdb_entry_unref (source->priv->current_station);
 				source->priv->current_station = NULL;
@@ -1756,10 +1931,6 @@ handle_station_response (RBLastfmSource *source, const char *body, RhythmDBEntry
 					track = (RhythmDBEntry *)i->data;
 					track_data = RHYTHMDB_ENTRY_GET_TYPE_DATA (track, RBLastfmTrackEntryData);
 
-					if (track_data->played == FALSE) {
-						source->priv->tracks_remaining--;
-					}
-
 					rhythmdb_entry_delete (source->priv->db, track);
 					rhythmdb_entry_unref (track);
 				}
@@ -1918,7 +2089,7 @@ xspf_entry_parsed (TotemPlParser *parser, const char *uri, GHashTable *metadata,
 	}
 
 	/* image URL and track auth ID are stored in entry type specific data */
-	value = g_hash_table_lookup (metadata, TOTEM_PL_PARSER_FIELD_IMAGE_URL);
+	value = g_hash_table_lookup (metadata, TOTEM_PL_PARSER_FIELD_IMAGE_URI);
 	if (value != NULL && (strcmp (value, LASTFM_NO_COVER_IMAGE) != 0)) {
 		track_data->image_url = g_strdup (value);
 	}
@@ -1928,7 +2099,7 @@ xspf_entry_parsed (TotemPlParser *parser, const char *uri, GHashTable *metadata,
 		track_data->track_auth = g_strdup (value);
 	}
 
-	value = g_hash_table_lookup (metadata, TOTEM_PL_PARSER_FIELD_DOWNLOAD_URL);
+	value = g_hash_table_lookup (metadata, TOTEM_PL_PARSER_FIELD_DOWNLOAD_URI);
 	if (value != NULL) {
 		track_data->download_url = g_strdup (value);
 		rb_debug ("track %s has a download url: %s", uri, track_data->download_url);
@@ -1936,7 +2107,8 @@ xspf_entry_parsed (TotemPlParser *parser, const char *uri, GHashTable *metadata,
 
 	/* what happens if it's already in there? need to use move_entry instead? */
 	rhythmdb_query_model_add_entry (source->priv->query_model, track_entry, -1);
-	source->priv->tracks_remaining++;
+
+	source->priv->last_entry = track_entry;
 }
 
 static gboolean
@@ -1952,6 +2124,11 @@ handle_playlist_response (RBLastfmSource *source, const char *body, RhythmDBEntr
 	gboolean ret = FALSE;
 	time_t now;
 	GValue value = {0,};
+
+	if (body == NULL) {
+		rb_debug ("didn't get a response");
+		return FALSE;
+	}
 	
 	/* until totem-pl-parser can parse playlists from in-memory data, we save it to a
 	 * temporary file.
@@ -2056,24 +2233,6 @@ queue_get_playlist_and_skip (RBLastfmSource *source, RhythmDBEntry *station)
 
 /* XMLRPC requests */
 
-/* can't be bothered implementing this stuff for libsoup 2.2. */
-#if defined(HAVE_LIBSOUP_2_2)
-
-static SoupMessage *
-create_action_request (RBLastfmSource *source, RhythmDBEntry *entry, const char *action)
-{
-	g_warning ("xmlrpc stuff not implemented for libsoup 2.2");
-	return NULL;
-}
-
-static void
-handle_xmlrpc_response (RBLastfmSource *source, const char *body, RhythmDBEntry *entry)
-{
-	/* nothing */
-}
-
-#else
-
 static SoupMessage *
 create_action_request (RBLastfmSource *source, RhythmDBEntry *entry, const char *action)
 {
@@ -2132,6 +2291,11 @@ handle_xmlrpc_response (RBLastfmSource *source, const char *body, RhythmDBEntry
 	GError *error = NULL;
 	GValue v = {0,};
 
+	if (body == NULL) {
+		rb_debug ("didn't get a response to an xmlrpc request");
+		return;
+	}
+
 	soup_xmlrpc_parse_method_response (body, strlen (body), &v, &error);
 	if (error != NULL) {
 		rb_debug ("got error in xmlrpc response: %s", error->message);
@@ -2142,7 +2306,6 @@ handle_xmlrpc_response (RBLastfmSource *source, const char *body, RhythmDBEntry
 
 	g_value_unset (&v);
 }
-#endif
 
 /* XMLRPC: banTrack */
 
