@@ -9,7 +9,7 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
  *  GStreamer plugins to be used and distributed together with GStreamer
  *  and Rhythmbox. This permission is above and beyond the permissions granted
  *  by the GPL license by which Rhythmbox is covered. If you modify this code
@@ -41,7 +41,6 @@
 #include <glib/gi18n.h>
 #include <gdk/gdk.h>
 
-#include "rb-soup-compat.h"
 #include <libsoup/soup.h>
 
 #include "rb-daap-hash.h"
@@ -54,6 +53,8 @@
 
 #define RB_DAAP_USER_AGENT "iTunes/4.6 (Windows; N)"
 
+#define ITUNES_7_SERVER "iTunes/7"
+
 static void      rb_daap_connection_dispose      (GObject *obj);
 static void      rb_daap_connection_set_property (GObject *object,
 						  guint prop_id,
@@ -357,11 +358,7 @@ build_message (RBDAAPConnection *connection,
 		char *token;
 
 		user_pass = g_strdup_printf ("%s:%s", priv->username, priv->password);
-#if defined(HAVE_LIBSOUP_2_4)
 		token = g_base64_encode ((guchar *)user_pass, strlen (user_pass));
-#else
-		token = soup_base64_encode (user_pass, strlen (user_pass));
-#endif
 		h = g_strdup_printf ("Basic %s", token);
 
 		g_free (token);
@@ -375,7 +372,7 @@ build_message (RBDAAPConnection *connection,
 		gchar hash[33] = {0};
 		gchar *no_daap_path = (gchar *)path;
 
-		if (g_strncasecmp (path, "daap://", 7) == 0) {
+		if (g_ascii_strncasecmp (path, "daap://", 7) == 0) {
 			no_daap_path = strstr (path, "/data");
 		}
 
@@ -417,10 +414,12 @@ connection_set_error_message (RBDAAPConnection *connection,
 			      const char       *message)
 {
 	/* FIXME: obtain a lock */
-	if (connection->priv->last_error_message != NULL) {
-		g_free (connection->priv->last_error_message);
+	g_free (connection->priv->last_error_message);
+	if (message != NULL) {
+		connection->priv->last_error_message = g_strdup (message);
+	} else {
+		connection->priv->last_error_message = NULL;
 	}
-	connection->priv->last_error_message = g_strdup (message);
 }
 
 typedef struct {
@@ -439,17 +438,13 @@ actual_http_response_handler (DAAPResponseData *data)
 	const char *encoding_header;
 	char *message_path;
 	int response_length;
+	gboolean compatible_server = TRUE;
 
 	priv = data->connection->priv;
 	structure = NULL;
 	encoding_header = NULL;
-#if defined(HAVE_LIBSOUP_2_4)
 	response = data->message->response_body->data;
 	response_length = data->message->response_body->length;
-#else
-	response = data->message->response.body;
-	response_length = data->message->response.length;
-#endif
 
 	message_path = soup_uri_to_string (soup_message_get_uri (data->message), FALSE);
 
@@ -459,7 +454,15 @@ actual_http_response_handler (DAAPResponseData *data)
 		  data->message->reason_phrase);
 
 	if (data->message->response_headers) {
+		const char *server;
+
 		encoding_header = soup_message_headers_get (data->message->response_headers, "Content-Encoding");
+
+		server = soup_message_headers_get (data->message->response_headers, "DAAP-Server");
+		if (server != NULL && strstr (server, ITUNES_7_SERVER) != NULL) {
+			rb_debug ("giving up.  we can't talk to %s", server);
+			compatible_server = FALSE;
+		}
 	}
 
 	if (SOUP_STATUS_IS_SUCCESSFUL (data->status) && encoding_header && strcmp (encoding_header, "gzip") == 0) {
@@ -532,7 +535,13 @@ actual_http_response_handler (DAAPResponseData *data)
 #endif
 	}
 
-	if (SOUP_STATUS_IS_SUCCESSFUL (data->status)) {
+	if (compatible_server == FALSE) {
+		/* leaving structure == NULL here causes the connection process
+		 * to fail at the first step.
+		 */
+		connection_set_error_message (data->connection,
+					      _("Rhythmbox is not able to connect to iTunes 7 shares"));
+	} else if (SOUP_STATUS_IS_SUCCESSFUL (data->status)) {
 		RBDAAPItem *item;
 
 		if (!rb_is_main_thread ()) {
@@ -596,16 +605,10 @@ actual_http_response_handler (DAAPResponseData *data)
 	g_free (data);
 }
 
-#if defined(HAVE_LIBSOUP_2_4)
 static void
 http_response_handler (SoupSession      *session,
 		       SoupMessage      *message,
 		       RBDAAPConnection *connection)
-#else
-static void
-http_response_handler (SoupMessage      *message,
-		       RBDAAPConnection *connection)
-#endif
 {
 	DAAPResponseData *data;
 	int response_length;
@@ -617,11 +620,7 @@ http_response_handler (SoupMessage      *message,
 
 	data = g_new0 (DAAPResponseData, 1);
 	data->status = message->status_code;
-#if defined(HAVE_LIBSOUP_2_4)
 	response_length = message->response_body->length;
-#else
-	response_length = message->response.length;
-#endif
 
 	g_object_ref (G_OBJECT (connection));
 	data->connection = connection;
@@ -1670,7 +1669,7 @@ rb_daap_connection_get_headers (RBDAAPConnection *connection,
 
 	priv->request_id++;
 
-	if (g_strncasecmp (uri, "daap://", 7) == 0) {
+	if (g_ascii_strncasecmp (uri, "daap://", 7) == 0) {
 		norb_daap_uri = strstr (uri, "/data");
 	}
 
@@ -1696,11 +1695,7 @@ rb_daap_connection_get_headers (RBDAAPConnection *connection,
 		char *token;
 
 		user_pass = g_strdup_printf ("%s:%s", priv->username, priv->password);
-#if defined(HAVE_LIBSOUP_2_4)
 		token = g_base64_encode ((guchar *)user_pass, strlen (user_pass));
-#else
-		token = soup_base64_encode (user_pass, strlen (user_pass));
-#endif
 		g_string_append_printf (headers, "Authentication: Basic %s\r\n", token);
 		g_free (token);
 		g_free (user_pass);
