@@ -37,10 +37,8 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
-#ifdef HAVE_HAL
 #include <libhal.h>
 #include <dbus/dbus.h>
-#endif
 
 #include "eel-gconf-extensions.h"
 #include "rb-psp-source.h"
@@ -104,6 +102,8 @@ rb_psp_source_new (RBShell *shell, GMount *mount)
 
 	source = RB_PSP_SOURCE (g_object_new (RB_TYPE_PSP_SOURCE,
 					  "entry-type", entry_type,
+					  "ignore-entry-type", RHYTHMDB_ENTRY_TYPE_INVALID,
+					  "error-entry-type", RHYTHMDB_ENTRY_TYPE_INVALID,
 					  "mount", mount,
 					  "shell", shell,
 					  "source-group", RB_SOURCE_GROUP_DEVICES,
@@ -271,14 +271,12 @@ rb_psp_source_create_playlists (RBGenericPlayerSource *source)
 	}
 }
 
-#ifdef HAVE_HAL
-
 static gboolean
-hal_udi_is_psp (const char *udi)
+volume_is_psp (GVolume *volume)
 {
 	LibHalContext *ctx;
 	DBusConnection *conn;
-	char *parent_udi;
+	char *parent_udi, *udi;
 	char *parent_name;
 	gboolean result;
 	DBusError error;
@@ -287,6 +285,7 @@ hal_udi_is_psp (const char *udi)
 	result = FALSE;
 	dbus_error_init (&error);
 
+	udi = NULL;
 	parent_udi = NULL;
 	parent_name = NULL;
 
@@ -304,6 +303,10 @@ hal_udi_is_psp (const char *udi)
 	if (!libhal_ctx_init (ctx, &error) || dbus_error_is_set (&error))
 		goto end;
 
+	udi = rb_gvolume_get_udi (volume, ctx);
+	if (udi == NULL)
+		goto end;
+
 	inited = TRUE;
 	parent_udi = libhal_device_get_property_string (ctx, udi,
 			"info.parent", &error);
@@ -320,6 +323,7 @@ hal_udi_is_psp (const char *udi)
 	}
 
 end:
+	g_free (udi);
 	g_free (parent_udi);
 	g_free (parent_name);
 
@@ -339,37 +343,20 @@ end:
 
 	return result;
 }
-#endif
 
 gboolean
 rb_psp_is_mount_player (GMount *mount)
 {
-#ifndef HAVE_HAL
-	GFile *music_dir;
-#else
 	GVolume *volume;
-#endif
-	gboolean result = FALSE;
+	gboolean result;
 
-#ifndef HAVE_HAL
-	music_dir = find_music_dir (mount);
-	if (music_dir != NULL) {
-		g_object_unref (music_dir);
-		result = TRUE;
-	}
-#else
 	volume = g_mount_get_volume (mount);
-	if (volume != NULL) {
-		char *str;
+	if (volume == NULL)
+		return FALSE;
+
+	result = volume_is_psp (volume);
+	g_object_unref (volume);
 
-		str = g_volume_get_identifier (volume, G_VOLUME_IDENTIFIER_KIND_HAL_UDI);
-		if (str != NULL) {
-			result = hal_udi_is_psp (str);
-			g_free (str);
-		}
-		g_object_unref (volume);
-	}
-#endif
 	return result;
 }
 
